/**
 * 项目：互联网医疗
 * 模型分组：服务管理
 * 模型名称：就诊人信息表
 * @Author: xiongwei
 * @Date: 2023-09-05 09:42:00
 */

package com.xwd.hospital.server.rest;

import java.util.List;
import java.util.Arrays;

import cn.dev33.satoken.stp.SaLoginModel;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.IdcardUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.Query;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import com.xwd.hospital.server.annotation.ApiCommon;
import com.xwd.hospital.server.annotation.AuthUser;
import com.xwd.hospital.server.base.BaseController;
import com.xwd.hospital.server.domain.User;
import com.xwd.hospital.server.enums.YesNoEnum;
import com.xwd.hospital.server.rest.res.ApiCode;
import com.xwd.hospital.server.domain.PatientInfo;
import com.xwd.hospital.server.rest.req.PatientInfoParam;
import com.xwd.hospital.server.rest.res.ApiResponse;
import com.xwd.hospital.server.service.PatientInfoService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@RestController
@Tag(name = "PatientInfoController", description = "就诊人信息表管理")
@RequestMapping("/v1/pms/patient-info")
public class PatientInfoController extends BaseController<PatientInfoService, PatientInfo, PatientInfoParam> {

    @Resource
    public void setService(PatientInfoService service) {
        this.service = service;
    }

    /**
     * 新增
     *
     * @param entity 要新增的对象
     * @return 新增的对象
     */
    @PostMapping("/")
    @Operation(summary = "新增")
    @Override
    public ApiResponse<PatientInfo> add(@RequestBody PatientInfo entity, @AuthUser User user) {
        if(StringUtils.isEmpty(entity.getIdNo())){
            return ApiResponse.fail(ApiCode.FAILED.getCode(),"身份证号码验证不正确");
        }
        if(!IdcardUtil.isValidCard(entity.getIdNo())){
            return ApiResponse.fail(ApiCode.FAILED.getCode(),"身份证号码验证不正确");
        }
        if(!StringUtils.isEmpty(entity.getGuardianIdentificationNumber())){
            if(!IdcardUtil.isValidCard(entity.getGuardianIdentificationNumber())){
                return ApiResponse.fail(ApiCode.FAILED.getCode(),"监护人身份证号码验证不正确");
            }
        }
        PatientInfo one = this.service.getOne(Wrappers.<PatientInfo>lambdaQuery().eq(PatientInfo::getIdNo, entity.getIdNo())
                .eq(PatientInfo::getUserInfoId, entity.getUserInfoId()).eq(PatientInfo::getDeleteStatus, YesNoEnum.NO));
        if(null != one){
            return ApiResponse.fail(ApiCode.FAILED.getCode(),"此就诊人身份证号已添加");
        }
        entity.setEditorId(user.getId());
        entity.setEditorName(user.getUsername());
        entity.setDefault().validate(true);
        this.service.save(entity);
        return ApiResponse.ok(this.service.getById(entity.getId()));
    }

    /**
     * 批量新增
     *
     * @param entityList 要新增的对象
     * @return boolean 成功或失败
     */
    @PostMapping("/batch-save")
    @Operation(summary = "批量新增")
    @Override
    public ApiResponse<Boolean> batchSave(@RequestBody List<PatientInfo> entityList, @AuthUser User user) {
        entityList.forEach(entity -> {
            entity.setEditorId(user.getId());
            entity.setEditorName(user.getUsername());
            entity.setDefault().validate(true);
        });
        return ApiResponse.ok(this.service.saveBatch(entityList));
    }

    /**
     * 更新
     *
     * @param entity 要更新的对象
     * @return 更新后的对象
     */
    @PutMapping("/")
    @Operation(summary = "更新")
    @ApiCommon
    @Override
    public ApiResponse<PatientInfo> update(
            @RequestBody PatientInfo entity,
            @RequestParam(value = "updateAllFields", defaultValue = "false") boolean updateAllFields,
            @AuthUser User user) {
        if (updateAllFields) {
            PatientInfo patientInfo = this.service.getById(entity.getId());
            if(null == patientInfo){
                return ApiResponse.fail(ApiCode.FAILED.getCode(),"就诊人信息不存在");
            }
            if(patientInfo.getDefaultStatus() == YesNoEnum.NO && entity.getDefaultStatus() == YesNoEnum.YES){
                //将其他默认就诊人去除
                LambdaQueryWrapper<PatientInfo> qw = new QueryWrapper<PatientInfo>().lambda()
                        .eq(PatientInfo::getUserInfoId, patientInfo.getUserInfoId())
                        .eq(PatientInfo::getDefaultStatus, YesNoEnum.YES);
                PatientInfo one = this.service.getOne(qw);
                if(null != one){
                    one.setDefaultStatus(YesNoEnum.NO);
                    this.service.saveOrUpdate(one);
                }
            }
            this.service.updateAllFieldsById(entity);
        } else {
            PatientInfo patientInfo = this.service.getById(entity.getId());
            if(null == patientInfo){
                return ApiResponse.fail(ApiCode.FAILED.getCode(),"就诊人信息不存在");
            }
            if(patientInfo.getDefaultStatus() == YesNoEnum.NO && entity.getDefaultStatus() == YesNoEnum.YES){
                //将其他默认就诊人去除
                LambdaQueryWrapper<PatientInfo> qw = new QueryWrapper<PatientInfo>().lambda()
                        .eq(PatientInfo::getUserInfoId, patientInfo.getUserInfoId())
                        .eq(PatientInfo::getDefaultStatus, YesNoEnum.YES);
                PatientInfo one = this.service.getOne(qw);
                if(null != one){
                    one.setDefaultStatus(YesNoEnum.NO);
                    this.service.saveOrUpdate(one);
                }
            }
            this.service.updateById(entity);
        }
        return ApiResponse.ok(this.service.getById(entity.getId()));
    }

    /**
     * 批量新增
     *
     */
    @PostMapping("/setDefault/{patientId}")
    @Operation(summary = "设为默认")
    public ApiResponse<Boolean> setDefault(@PathVariable("patientId") Long patientId) {
        PatientInfo patientInfo = this.service.getById(patientId);
        if(null == patientInfo){
            return ApiResponse.fail(ApiCode.FAILED.getCode(),"就诊人信息不存在");
        }
        patientInfo.setDefaultStatus(YesNoEnum.YES);
        this.service.saveOrUpdate(patientInfo);
        LambdaQueryWrapper<PatientInfo> qw = new QueryWrapper<PatientInfo>().lambda()
                .eq(PatientInfo::getUserInfoId, patientInfo.getUserInfoId())
                .eq(PatientInfo::getDefaultStatus, YesNoEnum.YES);
        PatientInfo one = this.service.getOne(qw);
        if(null != one){
            one.setDefaultStatus(YesNoEnum.NO);
            this.service.saveOrUpdate(one);
        }
        return ApiResponse.ok(true);
    }


}
